/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************
 *
 *                       MSP430 CODE EXAMPLE DISCLAIMER
 *
 * MSP430 code examples are self-contained low-level programs that typically
 * demonstrate a single peripheral function or device feature in a highly
 * concise manner. For this the code may rely on the device's power-on default
 * register values and settings such as the clock configuration and care must
 * be taken when combining code from several examples to avoid potential side
 * effects. Also see www.ti.com/grace for a GUI- and www.ti.com/msp430ware
 * for an API functional library-approach to peripheral configuration.
 *
 * --/COPYRIGHT--*/
//******************************************************************************
//  MSP430FR235x Demo: Measuring current using ADC and SAC_L3
//
//  Description: Current is measured using the SAC_L3 as a transimpedance
//  amplifier.
//  The result is measured using the ADC which is triggered in software.
//  When the signal drops below half the output (15uA), LED1 turns on.
//  ACLK = XT1 = 32768 Hz, MCLK = SMCLK = default DCODIV ~1MHz.
//
//                MSP430FR231x
//             -------------------
//            |          P1.3/OA0+|---GND
//         /|\|                   |
//          | |       P1.1/OA0O/A1|------------------|----->Vout/ADC
//          | |                   |                  |
//          --|RST                |   |--C1=150pF----|
//            |          P1.2/OA0-|---|--R1=100KOhm--|
//            |                   |   |
//            |                   |   |--Ii
//            |                   |
//            |               P1.0|---> LED1 (turns on if Ii > 15uA)
//            |                   |
//
//   Luis Reynoso
//   Texas Instruments Inc.
//   Dec 2019
//   Built with IAR Embedded Workbench v7.12.1 & Code Composer Studio v9.0.1 
//******************************************************************************
#include <msp430.h>

unsigned int adcResult;

int main(void)
{
    WDTCTL = WDTPW + WDTHOLD;                // Stop watch dog timer

    // Configure LED1 (P1.0)
    P1DIR |= BIT0;                       // Set P1.0 direction as output
    P1OUT &= ~BIT0;                      // Set P1.0 output low

    // Disable GPIO power-on default high-impedance mode to activate
    // previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;

    P1SEL0 |= BIT1 + BIT2 + BIT3;       // Select P1.1 P1.2 P1.3 OA function
    P1SEL1 |= BIT1 + BIT2 + BIT3;       // Select P1.1 P1.2 P1.3 OA function


    SAC0OA |= NMUXEN + PMUXEN;          // Enable negative and positive input
    SAC0OA |= OAPM;                     // Select low speed and low power mode
    SAC0PGA = MSEL_1;                   // Buffer Mode (PGA floating)
    SAC0OA |= SACEN + OAEN;             // Enable SAC and OA

    // Configure ADC to sample SAC_OA output (A1)
    ADCCTL0 &= ~ADCENC;                 // Disable ADC
    ADCCTL0 = ADCSHT_2 | ADCON;         // ADCON, S&H=16 ADC clks
    ADCCTL1 = ADCSSEL_0 | ADCDIV_0 |    // ADCCLK = MODCLK, divide by 1
              ADCCONSEQ_0 |             // Single Channel, single conversion,
              ADCSHP | ADCSHS_0;        // Pulse sample mode, SW trigger
    ADCCTL2 = ADCRES_2;                 // 12-bit conversion results
    ADCMCTL0 = ADCINCH_1 |              // A1 ADC input select = SAC output
               ADCSREF_0;               // Vref = DVCC
    ADCIE = ADCIE0;                     // Enable ADC conv complete interrupt


    while(1)
    {
        ADCCTL0 |= ADCENC | ADCSC;          // Sampling and conversion start
        __bis_SR_register(LPM0_bits | GIE); // Enter LPM0, ADC_ISR will force exit
        __no_operation();                   // For debug only

        if (adcResult > 0x7FF)              // OA output > 1/2 VREF
        {
            P1OUT |= BIT0;                  // Set P1.0 LED on
        }
        else
        {
            P1OUT &= ~BIT0;                 // Clear P1.0 LED off
        }
        __delay_cycles(5000);
    }
}

// ADC interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=ADC_VECTOR
__interrupt void ADC_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(ADC_VECTOR))) ADC_ISR (void)
#else
#error Compiler not supported!
#endif
{
    switch(__even_in_range(ADCIV,ADCIV_ADCIFG))
    {
        case ADCIV_NONE:
            break;
        case ADCIV_ADCOVIFG:
            break;
        case ADCIV_ADCTOVIFG:
            break;
        case ADCIV_ADCHIIFG:
            break;
        case ADCIV_ADCLOIFG:
            break;
        case ADCIV_ADCINIFG:
            break;
        case ADCIV_ADCIFG:
            adcResult = ADCMEM0;                    // Read ADC memory
            __bic_SR_register_on_exit(LPM0_bits);   // Exit from LPM
            break;
        default:
            break;
    }
}
